/* File: test_index.cpp
   Author:  T.S. Norvell
   Update:  0. 10 June 1999
*/

/* Test jig for the update_index function.
*/

#include <cstdlib>
#include <ctime>
#include <iostream>  // Standard I/O

#include "ch_string.h"
#include "update_index.h" 

using namespace std ;

static ch_string ffwords = "www.fwords.com" ;
static ch_string fmusic = "www.music.com" ;
static ch_string funk = "www.unkious.com" ;
 
void makeTestDatabase( Function< ch_string, Set<ch_string> >  &database ) 
{
    // File 0 
    {   Set< ch_string> dataFile ;
        ch_string fileName =  ffwords ;
        dataFile.insert( "foo" ) ;
        dataFile.insert( "firkin" ) ;
        dataFile.insert( "farthing" ) ;
        dataFile.insert( "filtrum" ) ;
        dataFile.insert( "fizzle" ) ;
        dataFile.insert( "fjord" ) ;
        dataFile.insert( "funk" ) ;

        database.add( fileName, dataFile ) ;
    }

    // File 1 
    {   Set< ch_string> dataFile ;
        ch_string fileName = fmusic ;
        dataFile.insert( "funk" ) ;
        dataFile.insert( "rock" ) ;
        dataFile.insert( "jazz" ) ;
        dataFile.insert( "classical" ) ;
        dataFile.insert( "tap" ) ;
        dataFile.insert( "country" ) ;
        dataFile.insert( "punk" ) ;

        database.add( fileName, dataFile ) ;
    }

    // File 2 
    {   Set< ch_string> dataFile ;
        ch_string fileName = funk ;
        dataFile.insert( "funk" ) ;
        dataFile.insert( "flunk" ) ;
        dataFile.insert( "sunk" ) ;
        dataFile.insert( "spunk" ) ;
        dataFile.insert( "lunk" ) ;
        dataFile.insert( "hunk" ) ;
        dataFile.insert( "punk" ) ;

        database.add( fileName, dataFile ) ;
    }
}

void makeIndex( Function< ch_string, StringSet > database,
                Index                         &index,
                StringSet                     &all ) {

    // Process each file in the data base.
    while( ! database.empty() ) {
        ch_string fileName ;
        StringSet contents ;

        database.getAny( fileName, contents ) ;

        // Add the file name to the set of all filenames.
        all.insert( fileName ) ;

 
        bool ok = false ;
        update_index( index, fileName, contents, ok ) ;


        database.remove( fileName ) ; }
}

void printStringSet( StringSet S ) {
    cout << "{" ;
    if( ! S.empty() ) {
        while( S.size() > 1 ) {
            ch_string someMember = S.getAny() ;
            cout << someMember << ", " ;
            S.remove( someMember ) ; }
        cout << S.getAny() ; }
    cout << "}" ; }

void printFunct( Function< ch_string, StringSet>  database ) {
    while( ! database.empty() ) {
        ch_string fileName ;
        StringSet contents ;
        database.getAny( fileName, contents ) ;
        cout << fileName << " --> " ;
        printStringSet( contents ) ;
        cout << "\n" ;
        database.remove( fileName ) ; } }

// testCount --- the number of tests //
///////////////////////////////////////
int testCount() { return 3 ; }

// doTest  --- do one test //
/////////////////////////////
void doTest( int testNumber, int &result ) {
    try {
        bool pass = true ;
        cout << "Test " << testNumber << "\n" ;
        bool ok = false ;

        Function< ch_string, StringSet > database ;
        makeTestDatabase( database ) ;
 
        static bool printed = false ;
        if( ! printed ) {
            cout << "Database is:\n" ;
            printFunct( database ) ;
            cout << "\n" ;
            printed = true ; }

        switch( testNumber ) {

        case 0 : {
            Index index;
            update_index( index,
                          funk,
                          database.at(funk),
                          ok ) ;
            cout << "Your answer is: \n" ;
            printFunct( index ) ;
            Index myIndex ;
            StringSet funkSet ; funkSet.insert(funk) ;
            StringSet emptySet ;
            myIndex.add( "funk", funkSet ) ;
            myIndex.add( "flunk", funkSet ) ;
            myIndex.add( "sunk", funkSet ) ;
            myIndex.add( "spunk", funkSet ) ;
            myIndex.add( "lunk", funkSet ) ;
            myIndex.add( "hunk", funkSet ) ;
            myIndex.add( "punk", funkSet ) ;
            
            cout << "My answer is: \n" ;
            printFunct( myIndex ) ;

            pass = ok && index == myIndex ;
                           
        } break ;
 
        case 1 : {
            Index index;
            update_index( index,
                          funk,
                          database.at(funk),
                          ok ) ;
            update_index( index,
                          ffwords,
                          database.at(ffwords),
                          ok ) ;
            update_index( index,
                          fmusic,
                          database.at(fmusic),
                          ok ) ;
            cout << "Your answer is: \n" ;
            printFunct( index ) ;
            Index myIndex ;
            StringSet funk_Set ; funk_Set.insert(funk) ;
            StringSet fmusic_Set ; fmusic_Set.insert(fmusic) ;
            StringSet ffwords_Set ; ffwords_Set.insert(ffwords) ;

            StringSet funk_fmusic_Set ;
            funk_fmusic_Set.insert(funk) ;
            funk_fmusic_Set.insert(fmusic) ;
            StringSet funk_ffwords_Set ;
            funk_ffwords_Set.insert(funk) ;
            funk_ffwords_Set.insert(ffwords) ;
            StringSet fmusic_ffwords_Set ;
            fmusic_ffwords_Set.insert(fmusic) ;
            fmusic_ffwords_Set.insert(ffwords) ;

            StringSet all ;
            all.insert(funk) ;
            all.insert(fmusic) ;
            all.insert(ffwords) ;

            myIndex.add( "country", fmusic_Set) ;
            myIndex.add( "tap", fmusic_Set) ;
            myIndex.add( "classical", fmusic_Set) ;
            myIndex.add( "jazz", fmusic_Set) ;
            myIndex.add( "rock", fmusic_Set) ;
            myIndex.add( "fjord", ffwords_Set) ;
            myIndex.add( "fizzle", ffwords_Set) ;
            myIndex.add( "filtrum", ffwords_Set) ;
            myIndex.add( "farthing", ffwords_Set) ;
            myIndex.add( "firkin", ffwords_Set) ;
            myIndex.add( "foo", ffwords_Set) ;
            myIndex.add( "punk", funk_fmusic_Set) ;
            myIndex.add( "hunk", funk_Set) ;
            myIndex.add( "lunk", funk_Set) ;
            myIndex.add( "spunk", funk_Set) ;
            myIndex.add( "sunk", funk_Set) ;
            myIndex.add( "flunk", funk_Set) ;
            myIndex.add( "funk", all) ;
            
            cout << "My answer is: \n" ;
            printFunct( myIndex ) ;

            pass = ok && index == myIndex ;
    
        } break ;

        case 2 : { 
            // Try some empty sets.
            StringSet empty ;
            Index index;
            update_index( index, "hello", empty, ok ) ;
            update_index( index,
                          funk,
                          database.at(funk),
                          ok ) ;
            update_index( index, "bye", empty, ok ) ;
            cout << "Your answer is: \n" ;
            printFunct( index ) ;
            Index myIndex ;
            StringSet funkSet ; funkSet.insert(funk) ;
            StringSet emptySet ;
            myIndex.add( "funk", funkSet ) ;
            myIndex.add( "flunk", funkSet ) ;
            myIndex.add( "sunk", funkSet ) ;
            myIndex.add( "spunk", funkSet ) ;
            myIndex.add( "lunk", funkSet ) ;
            myIndex.add( "hunk", funkSet ) ;
            myIndex.add( "punk", funkSet ) ;
            
            cout << "My answer is: \n" ;
            printFunct( myIndex ) ;

            pass = ok && index == myIndex ;
    
        } break ;

        case 3 : {     
        } break ;

        case 4 : {     
        } break ;

        case 5 : {     
        } break ;

        case 6 : {    
        } break ;
       default : pass = false ;
        } 

        if( pass ) {
            cout << "Test " << testNumber << " passes\n" ;
            result = 0 ; }
        else {
            cout << "Test " << testNumber << " fails\n" ;
            result = 1 ; } }
    catch( ... ) {
        cerr << "Sorry an unanticipated exception occurred\n" ;
        result = 1 ; } }